==> CRF%2B%2B-0.42.tar.gz: CRF++-0.42/node.cpp <==
/*
  CRF++ -- Yet Another CRF toolkit
 
  $Id: node.cpp,v 1.4 2006/03/19 17:15:32 taku Exp $;

  Copyright (C) 2005 Taku Kudo <taku@chasen.org>

  This is free software with ABSOLUTELY NO WARRANTY.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "node.h"
#include "common.h"
#include <cmath>

namespace CRFPP
{
  void Node::calcAlpha ()
  {
    alpha = 0.0;
    for (Path_iterator it = lpath.begin(); it != lpath.end(); ++it)
      alpha = logsumexp (alpha, (*it)->cost + (*it)->lnode->alpha, (it == lpath.begin()));

    alpha += cost;
  }

  void Node::calcBeta ()
  {
    beta = 0.0;
    for (Path_iterator it = rpath.begin(); it != rpath.end(); ++it)
      beta = logsumexp (beta,  (*it)->cost + (*it)->rnode->beta, (it == rpath.begin()));

    beta += cost;
  }

  void Node::calcExpectation (double *expected, double Z, size_t size)
  {
    double c = std::exp (alpha + beta - cost - Z); 
    for (int *f = fvector; *f != -1; ++f) expected[*f + y] += c;
    for (Path_iterator it = lpath.begin(); it != lpath.end(); ++it) 
      (*it)->calcExpectation (expected, Z, size);
  }
}

==> CRF%2B%2B-0.43.tar.gz: CRF++-0.43/node.cpp <==
/*
  CRF++ -- Yet Another CRF toolkit

  $Id: node.cpp,v 1.6 2006/08/07 02:35:35 taku Exp $;

  Copyright(C) 2005 Taku Kudo <taku@chasen.org>

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or(at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "node.h"
#include "common.h"
#include <cmath>

namespace CRFPP {

  void Node::calcAlpha()
  {
    alpha = 0.0;
    for (Path_iterator it = lpath.begin(); it != lpath.end(); ++it)
      alpha = logsumexp(alpha,
                        (*it)->cost + (*it)->lnode->alpha,
                        (it == lpath.begin()));
    alpha += cost;
  }

  void Node::calcBeta()
  {
    beta = 0.0;
    for (Path_iterator it = rpath.begin(); it != rpath.end(); ++it)
      beta = logsumexp(beta,
                       (*it)->cost + (*it)->rnode->beta,
                       (it == rpath.begin()));
    beta += cost;
  }

  void Node::calcExpectation(double *expected, double Z, size_t size)
  {
    double c = std::exp(alpha + beta - cost - Z);
    for (int *f = fvector; *f != -1; ++f) expected[*f + y] += c;
    for (Path_iterator it = lpath.begin(); it != lpath.end(); ++it)
      (*it)->calcExpectation(expected, Z, size);
  }
}
